#!/usr/bin/perl -w

use strict;
use warnings;
use Schedule::Cron;
use Time::ParseDate;
use Test::More 'no_plan';

# Constants to define a time w/in the skipped hour defined by the local
# DST rules.
my $DST_MONTH = 2;
my $DST_MDAY = 11;
my $DST_YEAR = 2007;
my $DST_HOUR = 2;
my $DST_MIN = 10;
my $DST_SEC = 0;
my $DST_WDAY = 0;

# These deliberately repeat the first element to make wrapping easier
my @WEEKDAYS = (
  'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'
);
my @MONTHS = (
  'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec','Jan'
);

my $HOURLY_CRON = "$DST_MIN * * * *";
my $DAILY_CRON = "$DST_MIN $DST_HOUR * * *";
my $WEEKLY_CRON = "$DST_MIN $DST_HOUR * * $DST_WDAY";
my $MONTHLY_CRON = "$DST_MIN $DST_HOUR $DST_MDAY * *";

my $sc = new Schedule::Cron(\&dispatcher);

my ($expected, $last, $next, $now);

# First, try some non-DST tests
$last = make_date($DST_WDAY, $DST_MONTH, $DST_MDAY, $DST_HOUR + 1,
                  $DST_MIN, $DST_SEC, $DST_YEAR);
$now = parsedate($last);

$expected = make_date($DST_WDAY, $DST_MONTH, $DST_MDAY, $DST_HOUR + 2,
                      $DST_MIN, $DST_SEC, $DST_YEAR);
$next = $sc->get_next_execution_time($HOURLY_CRON, $now);
is(localtime($next), $expected, 'non-DST hourly update');

$expected = make_date($DST_WDAY + 1, $DST_MONTH, $DST_MDAY + 1, $DST_HOUR,
                      $DST_MIN, $DST_SEC, $DST_YEAR);
$next = $sc->get_next_execution_time($DAILY_CRON, $now);
is(localtime($next), $expected, 'non-DST daily update');

$expected = make_date($DST_WDAY, $DST_MONTH, $DST_MDAY + 7, $DST_HOUR,
                      $DST_MIN, $DST_SEC, $DST_YEAR);
$next = $sc->get_next_execution_time($WEEKLY_CRON, parsedate($last));
is(localtime($next), $expected, 'non-DST weekly update');

$expected = make_date($DST_WDAY, $DST_MONTH + 1, $DST_MDAY, $DST_HOUR,
                      $DST_MIN, $DST_SEC, $DST_YEAR);
$expected =~ s/^\S+\s+//; # Don't know the weekday
$next = $sc->get_next_execution_time($MONTHLY_CRON, parsedate($last));
like(localtime($next), qr/$expected/, 'non-DST monthly update');

# Now, try crons that will put us into the hour skipped due to DST
$last = make_date($DST_WDAY, $DST_MONTH, $DST_MDAY, $DST_HOUR - 1,
                  $DST_MIN, $DST_SEC, $DST_YEAR);
$now = parsedate($last);
$expected = make_date($DST_WDAY, $DST_MONTH, $DST_MDAY, $DST_HOUR + 1,
                      $DST_MIN, $DST_SEC, $DST_YEAR);
$next = $sc->get_next_execution_time($HOURLY_CRON, $now);
is(localtime($next), $expected, 'DST hourly update');

$last = make_date($DST_WDAY - 1, $DST_MONTH, $DST_MDAY - 1, $DST_HOUR,
                  $DST_MIN, $DST_SEC, $DST_YEAR);
$now = parsedate($last);
$expected = make_date($DST_WDAY + 1, $DST_MONTH, $DST_MDAY + 1, $DST_HOUR,
                      $DST_MIN, $DST_SEC, $DST_YEAR);
$next = $sc->get_next_execution_time($DAILY_CRON, $now);
is(localtime($next), $expected, 'DST daily update');

$last = make_date($DST_WDAY, $DST_MONTH, $DST_MDAY - 7, $DST_HOUR,
                  $DST_MIN, $DST_SEC, $DST_YEAR);
$now = parsedate($last);
$expected = make_date($DST_WDAY, $DST_MONTH, $DST_MDAY + 7, $DST_HOUR,
                      $DST_MIN, $DST_SEC, $DST_YEAR);
$next = $sc->get_next_execution_time($WEEKLY_CRON, $now);
is(localtime($next), $expected, 'DST weekly update');

$last = make_date($DST_WDAY, $DST_MONTH - 1, $DST_MDAY, $DST_HOUR,
                  $DST_MIN, $DST_SEC, $DST_YEAR);
$now = parsedate($last);
$expected = make_date($DST_WDAY, $DST_MONTH + 1, $DST_MDAY, $DST_HOUR,
                      $DST_MIN, $DST_SEC, $DST_YEAR);
$expected =~ s/^\S+\s+//; # Don't know the weekday
$next = $sc->get_next_execution_time($MONTHLY_CRON, $now);
like(localtime($next), qr/$expected/, 'DST monthly update');

sub dispatcher {
  # empty
}

sub make_date {
  my ($wday, $month, $mday, $hour, $min, $sec, $year) = @_;
  $wday = 6 if $wday < 0;
  $month = 11 if $month < 0;
  return sprintf('%s %s %02d %02d:%02d:%02d %04d',
                 $WEEKDAYS[$wday],$MONTHS[$month],$mday,$hour,$min,$sec,$year);
};
